package Mentat::Message::Convertor::IDEA;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Message::Convertor::IDEA - Short module description

=head1 SYNOPSIS

 use Mentat::Message::Convertor::IDEA;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Data::Convertor(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use NetAddr::IP;

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Data::Convertor;
use Mentat::Message::IDEA;
use Value::IP;
use Value::Timestamp;
use Value::Period;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#"\x00"  Binary / Generic
#"\x01"  Function
#"\x02"  Binary (Old)
#"\x03"  UUID
#"\x05"  MD5
#"\x80"  User defined
use constant SUBTYPE_BINARY => 0;

use constant IP   => 'ip';
use constant MIN  => 'min';
use constant MAX  => 'max';
use constant TYPE => 'type';

use constant IPV4 => 'IPV4';
use constant IPV6 => 'IPV6';

#-------------------------------------------------------------------------------
# Structures for MongoDB HASH -> IDEA conversions
#-------------------------------------------------------------------------------

use constant MHASH_NODES => {
    # src_mpath => [conversion, dst_mpath]

    '_id'      => 0,
    'ts'       => 0,
    'ts_u'     => 0,
    'msg_raw'  => 0,
    'class'    => 0,
    'msg_raw2' => 0,
    'class2'   => 0,
    
    'CreateTime'   => ['_convert_hash_datetime', 'CreateTime'],
    'DetectTime'   => ['_convert_hash_datetime', 'DetectTime'],
    'EventTime'    => ['_convert_hash_datetime', 'EventTime'],
    'CeaseTime'    => ['_convert_hash_datetime', 'CeaseTime'],
    'WinStartTime' => ['_convert_hash_datetime', 'WinStartTime'],
    'WinEndTime'   => ['_convert_hash_datetime', 'WinEndTime'],

    'Source/IP4/min' => ['_convert_hash_ip', IPV4()],
    'Source/IP6/min' => ['_convert_hash_ip', IPV6()],
    'Target/IP4/min' => ['_convert_hash_ip', IPV4()],
    'Target/IP6/min' => ['_convert_hash_ip', IPV6()],
    'Source/IP4/max' => 0,
    'Source/IP6/max' => 0,
    'Target/IP4/max' => 0,
    'Target/IP6/max' => 0,
    'Source/IP4/ip'  => 0,
    'Source/IP6/ip'  => 0,
    'Target/IP4/ip'  => 0,
    'Target/IP6/ip'  => 0,
    

    'Source/Port' => ['anything_to_int'],
    'Target/Port' => ['anything_to_int'],
    'Source/ASN'  => ['anything_to_int'],
    'Target/ASN'  => ['anything_to_int'],
    'Attach/Size' => ['anything_to_int'],
    'ConnCount'   => ['anything_to_int'],
    'FlowCount'   => ['anything_to_int'],
    'PacketCount' => ['anything_to_int'],
    'ByteCount'   => ['anything_to_int'],
};

use constant MHASH_ADDRESS_CONVERSIONS => {
    IPV4() => '_hob_to_ipv4str',
    IPV6() => '_hob_to_ipv6str',
};

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 1;
    @ISA     = ('Data::Convertor');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item convert_node_mongo($$$) [PUBLIC]

  Usage     : my $result = $instance->convert_node_mongo($mongo_idea_message, )
  Purpose   : Convert whole MongoDB optimal HASH into Mentat::Message::IDEA
  Arguments : HASH $mongo_idea_message - message to be converted
  Returns   : Mentat::Message::IDEA REFERENCE containing the converted message
  Throws    : Croaks, if not invoked on object

=cut

sub convert_node_mongo($$$)
{
    my $self = shift;
    confess "Instance method invoked on class" unless blessed($self);
    my ($mhash, $src_mpath, $value) = @_;
    confess "Data::Tree object must be given as first argument" unless blessed($mhash) and $mhash->isa('Data::Tree');

    print STDERR "Converting Mongo node: '$src_mpath' => '$value'\n" if $DEVEL;

    # The are no indexes in the path conversion specifications, so remove them first
    my $src_mpath_s = $src_mpath; $src_mpath_s =~ s/\[\d*\]//g;

    # Get the indexes
    my @indexes = ($src_mpath =~ /\[(\d+)\]/g);
    my $variables = {}; my $i = 0;
    map { $variables->{++$i} = $_; } @indexes;

    # Handle the MPath unknown to the convertor
    if (not exists MHASH_NODES->{$src_mpath_s}) {
        #$src_mpath =~ s/^Alert\/|^Heartbeat\///;
        return ([$src_mpath, $value]);
    }
    # Skip the given node
    if(not MHASH_NODES()->{$src_mpath_s}) {
        print STDERR "Disabled path '$src_mpath_s', skipping\n" if $DEVEL;
        return ();
    }

    ### Handle the MPaths known to the convertor
    # Fetch the conversion information
    my ($conversion, @args) = @{MHASH_NODES()->{$src_mpath_s}};
    $conversion = $conversion || 'anything_to_samething';
    print STDERR "Attempting conversion with '$conversion' on '$self': ".($self->can($conversion))."\n" if $DEVEL;

    use Class::Inspector;
    my $methods = Class::Inspector->methods('Mentat::Message::Convertor::IDEA');

    print STDERR "Methods are [@$methods]\n"; # reports nothing

    # Dispatching for conversions performed directly via Value::Convertor module
    unless ($self->can($conversion)) {
        print STDERR "Indirect conversion with '$conversion'\n" if $DEVEL;
        return $self->_convert_node($src_mpath, $value, $conversion, $variables, @args);
    }
    else {
        print STDERR "Direct conversion with '$conversion'\n" if $DEVEL;
        return $self->$conversion($mhash, $src_mpath, $value, $variables, @args);
    }
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Message::Convertor instance
#  Arguments :
#  Returns   : Mentat::Message::Convertor reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    return $self;
}

# _convert_hash_datetime($$$$$;@) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _convert_hash_datetime($$$$$;@)
{
    my $self = shift;
    confess "Instance method invoked on class" unless blessed($self);
    my ($idea, $src_mpath, $value, $variables, $type, @args) = @_;
    confess "DATETIME binary string must be given as argument" unless defined $value;

    print STDERR "Datetime conversion of: ".Dumper($value) if $DEVEL;
    
    my $str = Value::Convertor->nbi_to_datestr(Value::Convertor->bin_to_bigint($value));

    return (["$type", $str]);
}

# _convert_hash_ip($$$$$;@) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _convert_hash_ip($$$$$;@)
{
    my $self = shift;
    confess "Instance method invoked on class" unless blessed($self);
    my ($idea, $src_mpath, $value, $variables, $type, @args) = @_;
    #confess "Address must be given as argument" unless defined $value;
    #confess "Type must be given as argument" unless defined $type;
    #confess "Invalid address type '$type'" unless IDEA_ADDRESS_CONVERSIONS()->{$type};

    # Strip the min/max/ip chunk from src_mpath and retrieve the whole IP representation object
    $src_mpath =~ s\(/min|/max|/ip)$\\;
    $value = $idea->path_value($src_mpath);
    print STDERR "IP conversion of: ".Dumper($value) if $DEVEL;
    
    my $conversion = MHASH_ADDRESS_CONVERSIONS()->{$type};
    my $result = $self->_convert_with_conversion($conversion, $value, @args);
    my $dst_mpath = $src_mpath;

    return ([$dst_mpath, $result]);
}

#-------------------------------------------------------------------------------

# array of binary
sub _hob_to_ipv6str($)
{
    my $self = shift;
    confess "Instance method invoked on class" unless blessed($self);
    my ($ipv6bin,) = @_;

    # http://backpan.perl.org/authors/id/S/SW/SWETH/IPv4.pm
    # http://code.google.com/p/ipv6gen/

    if ($ipv6bin->{ip} and $ipv6bin->{min} and $ipv6bin->{max} and ($ipv6bin->{ip} eq $ipv6bin->{min}) and ($ipv6bin->{ip} eq $ipv6bin->{max})) {
        return Value::Convertor->bin_to_ipv6str($ipv6bin->{ip});
    }
    else {
        return Value::Convertor->bin_to_ipv6str($ipv6bin->{min});
        #return Value::Convertor->bigints_to_ipv6cidr(
        #    Value::Convertor->bin_to_ipv6int($ipv6bin->{min}),
        #    Value::Convertor->bin_to_ipv6int($ipv6bin->{max}),
        #);
    }
}

sub _hob_to_ipv4str($)
{
    my $self = shift;
    confess "Instance method invoked on class" unless blessed($self);
    my ($ipv4bin,) = @_;

    # http://backpan.perl.org/authors/id/S/SW/SWETH/IPv4.pm
    # http://code.google.com/p/ipv6gen/

    if ($ipv4bin->{ip} and $ipv4bin->{min} and $ipv4bin->{max} and ($ipv4bin->{ip} eq $ipv4bin->{min}) and ($ipv4bin->{ip} eq $ipv4bin->{max})) {
        return Value::Convertor->bin_to_ipv4str($ipv4bin->{ip});
    }
    else {
        return Value::Convertor->ints_to_ipv4cidr(
            Value::Convertor->bin_to_ipv4int($ipv4bin->{min}),
            Value::Convertor->bin_to_ipv4int($ipv4bin->{max}),
        );
    }
}

=pod

=back

=cut

1;
