package Mentat::Processor::Component::State;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Component::State - Mentat message source engine for pickup folder

=head1 SYNOPSIS

 use Mentat::Processor::Component::State;

=head1 DESCRIPTION

=head1 AUTHOR

Jan Mach
Cesnet, z.s.p.o
jan.mach@cesnet.cz
http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use DB_File;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:c_general);
use Mentat::Processor::Component;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA %STATE_STORAGE);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Component');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item pstate($;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub pstate($;$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    my ($key, $value) = @_;

    return $self->{STORAGE}->{$key};
}

=item pstate_set($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub pstate_set($$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    my ($key, $value) = @_;

    $self->{STORAGE}->{$key} = $value;
}

=item pstate_del($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub pstate_del($)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    my ($key,) = @_;

    delete $self->{STORAGE}->{$key};
}

=item pstate_list() [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub pstate_list()
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);

    return keys(%{$self->{STORAGE}});
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Croaks, if invoked on class
  Comments  :
  See Also  :

=cut

sub DESTROY()
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless ref $self;

    untie %STATE_STORAGE if $self->{STORAGE};

    delete $self->{STORAGE};
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    :
#  Comments  :
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    die "[STT] Persistent state storage file must be specified under configuration key 'state_storage'\n" unless exists $config{state_storage};

    tie(%STATE_STORAGE, "DB_File", $config{state_storage}) or die "Can't open persistent state storage file $config{state_storage}: $!\n";
    $self->{STORAGE} = \%STATE_STORAGE;

    Log::Loger->info(__PACKAGE__, "[CMP:State] Using file '$config{state_storage}' as persistent state storage");

    return $self;
}

=pod

=back

=cut

1;
