package Mentat::Processor::Engine;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine - Base class for processor engine implementations

=head1 SYNOPSIS

 use Mentat::Processor::Engine;

=head1 DESCRIPTION

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Core qw(:c_general);
use Mentat::Processor::Component;
use Mentat::Processor::Message;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Component');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item type() [PUBLIC]

  Usage     : my $name = $component->type();
  Purpose   : Get the type of the component (first chunk after Mentat::Processor::Protocol)
  Arguments : None
  Returns   : STRING $name lowercase component name
  Throws    : Dies, if not invoked on instance

=cut

sub type()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return 'engine';
}

=item name() [PUBLIC]

  Usage     : my $name = $component->name();
  Purpose   : Get the name of the component (last chunk of the class name)
  Arguments : None
  Returns   : STRING $name lowercase component name
  Throws    : Dies, if not invoked on instance

=cut

sub name()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Lazy-determine the name of the component (last part of the class name)
    unless ($self->{NAME}) {
        my $name = blessed($self);
        confess "Can't automatically determine the name of the engine" unless $name =~ /Mentat::Processor::Engine/;

        $name =~ s/Mentat::Processor::Engine//; $name =~ s/^:://;
        my @tmp = split(/:+/, $name);
        if (@tmp) {
            $self->{NAME} = lc($tmp[$#tmp]);
        }
        else {
            $self->{NAME} = Mentat::Processor::Component::BASE_NAME;
        }
    }
    return $self->{NAME};
}

#-------------------------------------------------------------------------------

=item startup(;$) [PUBLIC]

  Usage     : $instance->startup();
  Purpose   : Callback to allow the component perform arbitrary startup operations
  Arguments : HASH REFERENCE $config - startup configurations [OPTIONAL]
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance or on error

=cut

sub startup(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($config,) = @_;
    confess "Component startup configurations must be passed via HASH reference" if $config and ref $config ne 'HASH';
}

=item shutdown(;$) [PUBLIC]

  Usage     : $instance->shutdown();
  Purpose   : Callback to allow the component perform arbitrary shutdown operations
  Arguments : HASH REFERENCE $config - shutdown configurations [OPTIONAL]
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance or on error

=cut

sub shutdown(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($config,) = @_;
    confess "Component shutdown configurations must be passed via HASH reference" if $config and ref $config ne 'HASH';
}

#-------------------------------------------------------------------------------

=item message_create(;@) [PUBLIC]

  Usage     :
  Purpose   :
  Returns   :
  Arguments :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub message_create(;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return Mentat::Processor::Message->new(@_);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init(%) [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Component instance
#  Arguments : HASH %config - component configurations
#  Returns   : Mentat::Processor::Component reference
#  Throws    : Dies, if not invoked on instance
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init(%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    # Perform own configuration sanity checks
    die "[ENG] Constructor expects configurations as argument\n" unless %config;
    die "[ENG] Engine configuration 'bound_event' is missing\n" unless $config{bound_event};

    # Name of the event to which to bind the main processing handler
    $self->{BOUND_EVENT}   = $config{bound_event};
    $self->{BOUND_HANDLER} = ($config{bound_handler})?('eh_'.$config{bound_handler}):'eh_process';

    return $self;
}

# _get_events_emitted(;@) [PROTECTED]
#
#  Usage     : Used as a hook method from template method register()
#  Purpose   : Return list of events emited by this component
#  Arguments : LIST OF ARRAY REFERENCES @lists - event lists from parent classess [OPTIONAL]
#  Returns   : ARRAY REFERENCE $events containing event names as STRINGS
#  Throws    : Dies, if not invoked on instance

sub _get_events_emitted(;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @lists = @_;

    return $self->SUPER::_get_events_emitted([

    ], @lists);
}

# _get_events_handled(;@) [PROTECTED]
#
#  Usage     : Used as a hook method from template method register()
#  Purpose   : Return list of events and appropriate handlers for this component
#  Arguments : LIST OF ARRAY REFERENCES @lists - event handler lists from parent classess [OPTIONAL]
#  Returns   : ARRAY REFERENCE $handlers with internal structure [$event, $handler]
#                  STRING $event   - event name
#                  STRING $handler - event handler name
#  Throws    : Dies, if not invoked on instance

sub _get_events_handled(;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @lists = @_;

    return $self->SUPER::_get_events_handled([
        [$self->{BOUND_EVENT}, $self->{BOUND_HANDLER}],
    ], @lists);
}

=pod

=back

=cut

1;
