package Mentat::Storage;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Storage - Base module for all Mentat object storage engines

=head1 SYNOPSIS

 use Mentat::Storage;

 use vars qw(@ISA);
 @ISA = ('Mentat::Storage');

 # Now implement the mandatory interface

=head1 DESCRIPTION

This module is the base class for all Mentat::Storage engines.

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.6';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Storage->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Throws    : Dies, if invoked on object
  Comments  : Internally uses _init() method
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my $self = bless ({}, $class);
    $self->_init(@_);
    return $self;
}

=item insert($) [PUBLIC,ABSTRACT]

  Usage     : my ($record_id, $error) = $storage->insert($object);
  Purpose   : Insert the given object into storage
  Arguments : MIXED $object - object to be stored into storage
  Returns   : STRING $oid - ID of the inserted object on SUCCESS, undef on FAILURE
              STRING $error - undef on SUCCESS, error string on failure
  Throws    : Dies, if not invoked on instance
              Croaks on insert failure
  Comments  : This method dies in case of insert failure, it must be taken care of on higher levels !

=cut

sub insert($)
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item fetch($) [PUBLIC,ABSTRACT]

  Usage     : my $object = $storage->fetch($oid);
  Purpose   : Fetch the Mentat message with given ID from storage
  Arguments : STRING $oid - unique identifier of the object in storage
  Returns   : HASH REFERENCE $object reference on SUCCESS,
              undef if object with given ID was not found,
  Throws    : Dies, if not invoked on instance
              Croaks on query failure

=cut

sub fetch($)
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item update($$;$) [PUBLIC,ABSTRACT]

  Usage     : my $result = $storage->update($filter, $values, $options);
  Purpose   : Update the existing object in the storage
  Arguments : HASH REFERENCE $filter - Update filter [MANDATORY]
              HASH REFERENCE $values - Object values [MANDATORY]
              HASH REFERENCE $options - Update options [OPTIONAL]
  Returns   : HASH REFERENCE containing information about the operation result:
                 ok  => success flag
                 n   => number of affected items
                 err => error flag
  Throws    : Dies, if not invoked on instance

=cut

sub update($$;$)
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item delete($) [PUBLIC,ABSTRACT]

  Usage     : my $result = $storage->delete($oid);
  Purpose   : Delete the object with given ID from the storage
  Arguments : STRING $oid - unique identifier of the object in storage
  Returns   : HASH REFERENCE containing information about the operation result:
                 ok  => success flag
                 n   => number of affected items
                 err => error flag
  Throws    : Dies, if not invoked on instance
  Comments  : This operation should always remove only one object
  See Also  : For bulk delete operation see remove() method

=cut

sub delete($)
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item count(;$) [PUBLIC,ABSTRACT]

  Usage     : my $count = $storage->count(); or
              my $count = $storage->count($filter);
  Purpose   : Get the number of Mentat messages in storage
  Arguments : HASH REFERENCE $filter - Query filter [OPTIONAL]
  Returns   : INTEGER on SUCCESS
  Throws    : Dies, if not invoked on instance
              Dies on operation failure

=cut

sub count(;$)
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item find(;$$$$$) [PUBLIC,ABSTRACT]

  Usage     : my ($objects, $count) = $storage->find(); or
              my ($objects, $count) = $storage->find($filter);
  Purpose   : Find the object(s) according to the given filter
  Arguments : HASH REFERENCE $filter - Query filter [OPTIONAL]
              HASH REFERENCE $selection - Record attribute selection [OPTIONAL]
              HASH REFERENCE $sort - Sorting specification [OPTIONAL]
              INTEGER $limit - Maximum number of records to be returned [OPTIONAL]
              INTEGER $skip - Number of objects to skip from the beginning of the result set [OPTIONAL]
  Returns   : ARRAY REFERENCE containing objects
              INTEGER total count of objects
  Throws    : Dies, if not invoked on instance

=cut

sub find(;$$$$$)
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item remove(;$) [PUBLIC,ABSTRACT]

  Usage     : my $result = $storage->remove(); or
              my $result = $storage->remove($filter);
  Purpose   : Remove the objects(s) according to the given filter
  Arguments : HASH REFERENCE $filter - Filter for selecting items to be removed [OPTIONAL]
  Returns   : HASH REFERENCE containing information about the operation result:
                 ok  => success flag
                 n   => number of affected items
                 err => error flag
  Throws    : Dies, if not invoked on instance
              Croaks on query failure
  Comments  : This operation performs bulk delete operation
  See Also  : For single delete operation see delete() method
  Warning   : DELETES ALL DATA, IF GIVEN NO FILTER !!!

=cut

sub remove(;$)
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item disconnect() [PUBLIC,ABSTRACT]

  Usage     : $storage->disconnect();
  Purpose   : Close existing storage connection
  Arguments : NONE
  Returns   : UNDEF on SUCCESS, STRING $error on FAILURE
  Throws    : Dies, if not invoked on instance

=cut

sub disconnect()
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=item reconnect() [PUBLIC,ABSTRACT]

  Usage     : $storage->reconnect();
  Purpose   : Optionally close existing storage connection and reopen it
  Arguments : NONE
  Returns   : UNDEF on SUCCESS, STRING $error on FAILURE
  Throws    : Dies, if not invoked on instance

=cut

sub reconnect()
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on instance
#  Comments  :
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Method needs implementation in child class";
}

=pod

=back

=cut

1;
